#|

File        : ordinal-arith.lisp
Authors     : Matyas
Date created: 2002-04-27
Revision    : $Id: ordinal-arith.lisp,v 1.25 2003/01/22 16:34:48 sustik Exp sustik $

In this book the ordinal arithmetic book(s) developed by Pete and
Daron is extended in order to prove Dixon's lemma.

I started this work with Sandip who greatly helped by discussing the
proof structure and started the collect the ordinal lemmas.  With the
development of Pete and Daron's ordinal book released in v2-7 which
uses Cantor normal form and provides most of the necessary lemmas this
book has undergone a lot of changes.  The ultimate goal is that all
ordinal lemmas will become part of the released ordinal books.

|#

;(SET-MATCH-FREE-ERROR NIL)

(in-package "ACL2")

(include-book "../../../../ordinals/ordinal-definitions")
(include-book "../../../../ordinals/ordinal-addition")
(include-book "../../../../ordinals/ordinal-exponentiation")
(include-book "../../../../ordinals/ordinal-isomorphism")

(in-theory (disable op))
(in-theory (disable o<))
(in-theory (disable o+))
(in-theory (disable o^))
(in-theory (disable |b < a  =>  ~(a < b) / ~(a = b)|))
(in-theory (disable w^a-def))
(in-theory (disable |a^(b+c)  =  a^b * a^c|))

(defun binary-ord-min (x y)
  (if (e0-ord-< x y) x y))

(defmacro ord-min (x y &rest args)
  (cond ((endp args) `(binary-ord-min ,x ,y))
        (t `(binary-ord-min ,x (ord-min ,y ,(car args) ,@(cdr args))))))

(defmacro ord<= (x y)
  `(not (o< ,y ,x)))

(defthm o<-antireflexive
  (not (o< x x)))

(defthm o<-transitive
  (implies (and (o< x y)
		(o< y z))
	   (o< x z)))

(defthm o<-total
  (implies (and (not (o< x y))
		(not (o< y x))
		(op x)
		(op y))
	   (equal x y))
  :rule-classes :forward-chaining)

(defthm o<-antisymmetric
  (implies (o< x y)
	   (not (o< y x)))
  :hints (("Goal" :use (:instance o<-transitive (z x)))))

(defthm o<=-<-transitivity
  (implies (and (not (o< y x))
		(o< y z)
		(op x)
		(op z))
	   (o< x z))
  :hints (("Goal" :cases ((o< x y) (o< z x)))))

(defthm <-<=-transitivity
  (implies (and (o< x y)
                (not (o< z y))
                (op y)
                (op z))
           (o< x z)))

(defthm <=-transitivity
  (implies (and (not (o< y x))
		(not (o< z y))
		(op x)
		(op y)
		(op z))
	   (not (o< z x))))

(defthm technical-0
  (implies (and (op x)
		(op y)
		(op z)
		(o< x y))
	   (ord<= (o+ x z)
		  (o+ y z)))
  :hints (("Goal" :cases ((equal (o+ x z) (o+ y z))))))

(defthm technical-0.1
  (implies (and (op x)
		(op y)
		(op z)
		(ord<= x y))
	   (ord<= (o+ x z)
		  (o+ y z)))
  :hints (("Goal" :cases ((equal (o+ x z) (o+ y z)) (equal x y)))))

(defthm technical-1.0
  (implies (and (op x)
		(op y)
		(op z)
		(o< y z))
	   (o< (o+ x y)
	       (o+ x z))))

(defthm technical-1.2
  (implies (and (op x)
		(op y)
		(op z)
		(not (o< z y)))
	   (not (o< (o+ x z)
		    (o+ x y))))
  :hints (("Goal" :use technical-1.0)))

(defthm technical-7
  (implies (and (op x)
		(op y)
		(ord<= x y))
	   (ord<= (o^ (omega) x)
		   (o^ (omega) y)))
  :hints (("Goal" :cases ((o< x y)))))

(defthm technical-8
  (implies (op x)
	   (op (o^ (omega) x))))

(defthm technical-9
  (implies (and (op x1)
		(op x2)
		(op y1)
		(op y2)
		(ord<= x1 x2)
		(ord<= y1 y2))
	   (ord<= (o+ x1 y1)
		  (o+ x2 y2)))
  :hints (("Goal" :use (:instance <=-transitivity
				  (x (o+ x1 y1))
				  (y (o+ x2 y1))
				  (z (o+ x2 y2))))))

(defthm technical-11
  (implies (and (op x)
		(op y1)
		(op y2)
		(ord<= y1 y2))
	   (ord<= y1 (o+ x y2)))
  :hints (("Goal" :use (:instance technical-9
				  (x1 0)
				  (x2 x)))))

(defthm technical-12a
  (implies (and (op x)
		(op y)
		(o< x y))
	   (o< (o^ (omega) x) (o^ (omega) y))))

(defthm technical-15
  (implies (and (op x)
		(op y)
		(or (not (equal 0 x))
		    (not (equal 0 y))))
	   (equal (equal (o+ x y) 0) NIL)))

(defthm | x < y => x+1 <= y |
  (implies (and (op x)
		(op y)
		(o< x y))
	   (ord<= (o+ x 1) y)))

;; P&D has this as a forward chaining rule.
(defthm |x <= y & y <= x => x = y|
  (implies (and (op x)
		(op y)
		(ord<= x y)
		(ord<= y x))
	   (equal (equal x y) T)))

(defthm |ord: x+y = x+z => y=z|
  (implies (and (op x)
		(op y)
		(op z))
	   (equal (equal (o+ x y)
			 (o+ x z))
		  (equal y z)))
  :hints (("Goal"
	   :use (technical-1.0
		 (:instance technical-1.0 (y z) (z y))))))

(defthm |ord: x = x+z => 0=z|
  (implies (and (op x)
		(op z))
	   (equal (equal x (o+ x z))
		  (equal 0 z)))
  :hints (("Goal"
	   :use (:instance |ord: x+y = x+z => y=z| (y 0)))))

(defthm |ord: y << x & x = y+z => x=z)|
  (implies (and (op x)
		(op y)
		(op z)
		(o< (first-exp y) (first-exp x)))
	   (equal (equal x (o+ y z))
		  (equal x z))))

(defthm technical-nat-<-omega
  (implies (natp n)
	   (o< n (omega))))

(defthm technical-nat-<=-omega
  (implies (natp n)
	   (ord<= n (omega))))

(defthm technical-<-implies-o<
  (implies (and (natp x)
		(natp y)
		(< x y))
	   (o< x y))
  :hints (("Goal" :expand (o< x y))))

(defthm technical-<=-implies-ord-<=
  (implies (and (natp x)
		(natp y)
		(<= x y))
	   (ord<= x y))
  :hints (("Goal" :cases ((< x y)))))
